@@ -166,6 +166,11 @@ int libspectrum_split_to_48k_pages( libspectrum_snap *snap,
 
 libspectrum_snap* libspectrum_snap_alloc_internal( void );
 
+libspectrum_error
+libspectrum_snap_write_buffer( libspectrum_buffer *buffer, int *out_flags,
+                               libspectrum_snap *snap, libspectrum_id_t type,
+                               libspectrum_creator *creator, int in_flags );
+
 /* Format specific snapshot routines */
 
 libspectrum_error
@@ -175,8 +180,8 @@ libspectrum_error
 internal_sna_read( libspectrum_snap *snap,
 		   const libspectrum_byte *buffer, size_t buffer_length );
 libspectrum_error
-libspectrum_sna_write( libspectrum_byte **buffer, size_t *length,
-		       int *out_flags, libspectrum_snap *snap, int in_flags );
+libspectrum_sna_write( libspectrum_buffer *buffer, int *out_flags,
+                       libspectrum_snap *snap, int in_flags );
 libspectrum_error
 libspectrum_snp_read( libspectrum_snap *snap,
 		      const libspectrum_byte *buffer, size_t buffer_length );
@@ -187,15 +192,15 @@ libspectrum_error
 libspectrum_szx_read( libspectrum_snap *snap,
 		      const libspectrum_byte *buffer, size_t buffer_length );
 libspectrum_error
-libspectrum_szx_write( libspectrum_byte **buffer, size_t *length,
-		       int *out_flags, libspectrum_snap *snap,
-		       libspectrum_creator *creator, int in_flags );
+libspectrum_szx_write( libspectrum_buffer *buffer, int *out_flags,
+                       libspectrum_snap *snap, libspectrum_creator *creator,
+                       int in_flags );
 libspectrum_error
 internal_z80_read( libspectrum_snap *snap,
 		   const libspectrum_byte *buffer, size_t buffer_length );
 libspectrum_error
-libspectrum_z80_write2( libspectrum_byte **buffer, size_t *length,
-			int *out_flags, libspectrum_snap *snap, int in_flags );
+libspectrum_z80_write2( libspectrum_buffer *buffer, int *out_flags,
+                        libspectrum_snap *snap, int in_flags );
 libspectrum_error
 libspectrum_zxs_read( libspectrum_snap *snap,
 		      const libspectrum_byte *buffer, size_t buffer_length );
@@ -231,16 +236,15 @@ internal_tap_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
 		   const size_t length, libspectrum_id_t type );
 
 libspectrum_error
-internal_tap_write( libspectrum_byte **buffer, size_t *length,
-		    libspectrum_tape *tape, libspectrum_id_t type );
+internal_tap_write( libspectrum_buffer *buffer, libspectrum_tape *tape,
+                    libspectrum_id_t type );
 
 libspectrum_error
 internal_tzx_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
 		   const size_t length );
 
 libspectrum_error
-internal_tzx_write( libspectrum_byte **buffer, size_t *length,
-		    libspectrum_tape *tape );
+internal_tzx_write( libspectrum_buffer *buffer, libspectrum_tape *tape );
 
 libspectrum_error
 internal_warajevo_read( libspectrum_tape *tape,
@@ -255,8 +259,7 @@ libspectrum_csw_read( libspectrum_tape *tape,
                       const libspectrum_byte *buffer, size_t length );
 
 libspectrum_error
-libspectrum_csw_write( libspectrum_byte **buffer, size_t *length,
-                       libspectrum_tape *tape );
+libspectrum_csw_write( libspectrum_buffer *buffer, libspectrum_tape *tape );
 
 libspectrum_error
 libspectrum_wav_read( libspectrum_tape *tape, const char *filename );
@@ -311,6 +314,18 @@ libspectrum_slist_cleanup( void );
 
 void
 libspectrum_hashtable_cleanup( void );
+
+#ifdef HAVE_STDATOMIC_H
+#include <stdatomic.h>
+
+void
+atomic_lock( atomic_char *lock_ptr );
+
+void
+atomic_unlock( atomic_char *lock_ptr );
+
+#endif				/* #ifdef HAVE_STDATOMIC_H */
+
 #endif				/* #ifndef HAVE_LIB_GLIB */
 
 #endif				/* #ifndef LIBSPECTRUM_INTERNALS_H */
