@@ -148,27 +148,25 @@ rzx_read_sign_end( libspectrum_rzx *rzx, const libspectrum_byte **ptr,
 		   const libspectrum_byte *end );
 
 static void
-rzx_write_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		  size_t *length, ptrdiff_t *sign_offset, int sign );
+rzx_write_header( libspectrum_buffer *buffer, int sign );
 static void
-rzx_write_creator( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		   size_t *length, libspectrum_creator *creator );
+rzx_write_creator( libspectrum_buffer *buffer, libspectrum_buffer *block_data,
+                   libspectrum_creator *creator );
 static libspectrum_error
-rzx_write_snapshot( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		    size_t *length, libspectrum_snap *snap,
-		    libspectrum_id_t snap_format,
-		    libspectrum_creator *creator, int compress );
+rzx_write_snapshot( libspectrum_buffer *buffer, libspectrum_buffer *block_data,
+                    libspectrum_snap *snap, libspectrum_id_t snap_format,
+                    libspectrum_creator *creator, int compress );
+static void
+rzx_write_input( input_block_t *block, libspectrum_buffer *buffer,
+                 libspectrum_buffer *block_data, int compress );
 static libspectrum_error
-rzx_write_input( input_block_t *block, libspectrum_byte **buffer,
-		 libspectrum_byte **ptr, size_t *length, int compress );
-static libspectrum_error
-rzx_write_signed_start( libspectrum_byte **buffer, libspectrum_byte **ptr,
-			size_t *length, libspectrum_rzx_dsa_key *key,
+rzx_write_signed_start( libspectrum_buffer *buffer,
+                        libspectrum_buffer *block_data,
+                        libspectrum_rzx_dsa_key *key,
 			libspectrum_creator *creator );
 static libspectrum_error
-rzx_write_signed_end( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		      size_t *length, ptrdiff_t sign_offset,
-		      libspectrum_rzx_dsa_key *key );
+rzx_write_signed_end( libspectrum_buffer *buffer, libspectrum_buffer *block_data,
+                      libspectrum_rzx_dsa_key *key );
 
 /* The signature used to identify .rzx files */
 static const char * const rzx_signature = "RZX!";
@@ -288,7 +286,6 @@ libspectrum_error
 libspectrum_rzx_stop_input( libspectrum_rzx *rzx )
 {
   rzx->current_input = NULL;
-
   return LIBSPECTRUM_ERROR_NONE;
 }
 
@@ -296,10 +293,8 @@ libspectrum_error
 libspectrum_rzx_add_snap( libspectrum_rzx *rzx, libspectrum_snap *snap, int automatic )
 {
   rzx_block_t *block;
-  libspectrum_error error;
 
-  error = libspectrum_rzx_stop_input( rzx );
-  if( error ) return error;
+  libspectrum_rzx_stop_input( rzx );
 
   block_alloc( &block, LIBSPECTRUM_RZX_SNAPSHOT_BLOCK );
 
@@ -339,8 +334,7 @@ libspectrum_rzx_rollback( libspectrum_rzx *rzx, libspectrum_snap **snap )
   }
 
   if( rzx->current_input ) {
-    libspectrum_error error;
-    error = libspectrum_rzx_stop_input( rzx ); if( error ) return error;
+    libspectrum_rzx_stop_input( rzx );
   }
 
   /* Delete all blocks after the snapshot */
@@ -378,8 +372,7 @@ libspectrum_rzx_rollback_to( libspectrum_rzx *rzx, libspectrum_snap **snap,
   }
 
   if( rzx->current_input ) {
-    libspectrum_error error;
-    error = libspectrum_rzx_stop_input( rzx ); if( error ) return error;
+    libspectrum_rzx_stop_input( rzx );
   }
 
   /* Delete all blocks after the snapshot */
@@ -619,7 +612,6 @@ libspectrum_rzx_free( libspectrum_rzx *rzx )
   g_slist_foreach( rzx->blocks, block_free_wrapper, NULL );
   g_slist_free( rzx->blocks );
   libspectrum_free( rzx );
-
   return LIBSPECTRUM_ERROR_NONE;
 }
 
@@ -1256,17 +1248,20 @@ libspectrum_rzx_write( libspectrum_byte **buffer, size_t *length,
 		       libspectrum_rzx_dsa_key *key )
 {
   libspectrum_error error;
-  libspectrum_byte *ptr = *buffer;
   GSList *list;
-  ptrdiff_t sign_offset;
+  libspectrum_byte *ptr = *buffer;
+  libspectrum_buffer *new_buffer = libspectrum_buffer_alloc();
+  libspectrum_buffer *block_data = libspectrum_buffer_alloc();
 
-  rzx_write_header( buffer, &ptr, length, &sign_offset, key ? 1 : 0 );
-
-  if( creator ) rzx_write_creator( buffer, &ptr, length, creator );
+  if( creator ) rzx_write_creator( new_buffer, block_data, creator );
 
   if( key ) {
-    error = rzx_write_signed_start( buffer, &ptr, length, key, creator );
-    if( error != LIBSPECTRUM_ERROR_NONE ) return error;
+    error = rzx_write_signed_start( new_buffer, block_data, key, creator );
+    if( error != LIBSPECTRUM_ERROR_NONE ) {
+      libspectrum_buffer_free( new_buffer );
+      libspectrum_buffer_free( block_data );
+      return error;
+    }
   }
 
   for( list = rzx->blocks; list; list = list->next ) {
@@ -1276,18 +1271,22 @@ libspectrum_rzx_write( libspectrum_byte **buffer, size_t *length,
     switch( block->type ) {
 
     case LIBSPECTRUM_RZX_SNAPSHOT_BLOCK:
-      error = rzx_write_snapshot( buffer, &ptr, length, block->types.snap.snap,
-				  snap_format, creator, compress );
-      if( error != LIBSPECTRUM_ERROR_NONE ) return error;
+      error = rzx_write_snapshot( new_buffer, block_data,
+                                  block->types.snap.snap, snap_format, creator,
+                                  compress );
+      if( error != LIBSPECTRUM_ERROR_NONE ) {
+        libspectrum_buffer_free( new_buffer );
+        libspectrum_buffer_free( block_data );
+        return error;
+      }
       break;
 
     case LIBSPECTRUM_RZX_INPUT_BLOCK:
       /* z80 snapshots can't safely store an intermediate state */
       snap_format = LIBSPECTRUM_ID_SNAPSHOT_SZX;
 
-      error = rzx_write_input( &( block->types.input ), buffer, &ptr, length,
-			       compress );
-      if( error != LIBSPECTRUM_ERROR_NONE ) return error;
+      rzx_write_input( &( block->types.input ), new_buffer, block_data,
+                       compress );
       break;
 
     case LIBSPECTRUM_RZX_CREATOR_BLOCK:
@@ -1299,101 +1298,146 @@ libspectrum_rzx_write( libspectrum_byte **buffer, size_t *length,
   }
 
   if( key ) {
-    error = rzx_write_signed_end( buffer, &ptr, length, sign_offset, key );
-    if( error != LIBSPECTRUM_ERROR_NONE ) return error;
+    error = rzx_write_signed_end( new_buffer, block_data, key );
+    if( error != LIBSPECTRUM_ERROR_NONE ) {
+      libspectrum_buffer_free( new_buffer );
+      libspectrum_buffer_free( block_data );
+      return error;
+    }
   }
   
-  /* *length is the allocated size; we want to return how much is used */
-  *length = ptr - *buffer;
+  rzx_write_header( block_data, key ? 1 : 0 );
+  libspectrum_buffer_append( buffer, length, &ptr, block_data );
+  libspectrum_buffer_free( block_data );
+
+  libspectrum_buffer_append( buffer, length, &ptr, new_buffer );
+  libspectrum_buffer_free( new_buffer );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
 
 static void
-rzx_write_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		  size_t *length, ptrdiff_t *sign_offset, int sign )
+rzx_write_header( libspectrum_buffer *buffer, int sign )
 {
-  size_t signature_length = strlen( rzx_signature );
+  libspectrum_buffer_write( buffer, rzx_signature, strlen( rzx_signature ) );
 
-  libspectrum_make_room( buffer, strlen( rzx_signature ) + 6, ptr, length );
-  memcpy( *ptr, rzx_signature, signature_length ); *ptr += signature_length;
-
-  *(*ptr)++ = 0;		/* Major version number */
+  libspectrum_buffer_write_byte( buffer, 0 );	/* Major version number */
 
   /* Flags */
 #ifdef HAVE_GCRYPT_H
-  *(*ptr)++ = sign ? 13 : 12;	/* Minor version number: 12 if we're not
-				   signing, 13 if we are */
-  libspectrum_write_dword( ptr, sign ? 0x01 : 0x00 );
-
-  /* Store where to start signing data from */
-  *sign_offset = *ptr - *buffer;
-
+  /* Minor version number: 12 if we're not signing, 13 if we are */
+  libspectrum_buffer_write_dword( buffer, sign ? 0x01 : 0x00 );
 #else				/* #ifdef HAVE_GCRYPT_H */
-  *(*ptr)++ = 12;		/* Minor version number */
-  libspectrum_write_dword( ptr, 0 );
+  libspectrum_buffer_write_byte( buffer, 12 );	/* Minor version number */
+  libspectrum_buffer_write_dword( buffer, 0 );
 #endif				/* #ifdef HAVE_GCRYPT_H */
 }
 
 static void
-rzx_write_creator( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		   size_t *length, libspectrum_creator *creator )
+rzx_write_block_header( libspectrum_buffer *dest, libspectrum_buffer *src,
+                        libspectrum_byte type )
 {
-  size_t custom_length, block_length;
+  size_t block_length = libspectrum_buffer_get_data_size( src ) +
+                      sizeof( libspectrum_dword ) + sizeof( libspectrum_byte );
 
-  custom_length = libspectrum_creator_custom_length( creator );
-  block_length = 29 + custom_length;
+  libspectrum_buffer_write_byte( dest, type );
+  libspectrum_buffer_write_dword( dest, block_length );
 
-  libspectrum_make_room( buffer, block_length, ptr, length );
+  libspectrum_buffer_write_buffer( dest, src );
 
-  *(*ptr)++ = LIBSPECTRUM_RZX_CREATOR_BLOCK;
-  libspectrum_write_dword( ptr, block_length );	/* Block length */
+  libspectrum_buffer_clear( src );
+}
 
-  memcpy( *ptr, libspectrum_creator_program( creator ), 20 ); (*ptr) += 20;
+static void
+rzx_write_creator( libspectrum_buffer *buffer, libspectrum_buffer *block_data,
+                   libspectrum_creator *creator )
+{
+  size_t custom_length = libspectrum_creator_custom_length( creator );
 
-  libspectrum_write_word( ptr, libspectrum_creator_major( creator ) );
-  libspectrum_write_word( ptr, libspectrum_creator_minor( creator ) );
+  libspectrum_buffer_write( block_data, libspectrum_creator_program( creator ),
+                            20 );
+
+  libspectrum_buffer_write_word( block_data, libspectrum_creator_major( creator ) );
+  libspectrum_buffer_write_word( block_data, libspectrum_creator_minor( creator ) );
 
   if( custom_length ) {
-    memcpy( *ptr, libspectrum_creator_custom( creator ), custom_length );
-    (*ptr) += custom_length;
+    libspectrum_buffer_write( block_data,
+                              libspectrum_creator_custom( creator ),
+                              custom_length );
   }
+
+  rzx_write_block_header( buffer, block_data, LIBSPECTRUM_RZX_CREATOR_BLOCK );
 }
 
 static libspectrum_error
-rzx_write_snapshot( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		    size_t *length, libspectrum_snap *snap,
-		    libspectrum_id_t snap_format,
-		    libspectrum_creator *creator, int compress )
+rzx_compress( libspectrum_buffer *dest, libspectrum_buffer *src, int *compress )
 {
+#ifdef HAVE_ZLIB_H
+  libspectrum_byte *out_data = libspectrum_buffer_get_data( src );
+  size_t out_length = libspectrum_buffer_get_data_size( src );
+  libspectrum_byte *compressed_data = NULL;
   libspectrum_error error;
-  libspectrum_byte *snap_buffer = NULL; size_t snap_length;
-  libspectrum_byte *gzsnap = NULL; size_t gzlength = 0;
+#endif
+
+  if( !(*compress) ) {
+    libspectrum_buffer_write_buffer( dest, src );
+    return LIBSPECTRUM_ERROR_NONE;
+  }
+
+#ifdef HAVE_ZLIB_H
+  error = libspectrum_zlib_compress( out_data, out_length, &compressed_data,
+                                     &out_length );
+  if( error != LIBSPECTRUM_ERROR_NONE || 
+      out_length >= libspectrum_buffer_get_data_size( src ) ) {
+    *compress = 0;
+    return error;
+  }
+
+  libspectrum_buffer_write( dest, compressed_data, out_length );
+  libspectrum_free( compressed_data );
+
+  return LIBSPECTRUM_ERROR_NONE;
+
+#else				/* #ifdef HAVE_ZLIB_H */
+
+  libspectrum_print_error( LIBSPECTRUM_ERROR_UNKNOWN,
+                           "rzx_compress: compression needs zlib" );
+  return LIBSPECTRUM_ERROR_UNKNOWN;
+
+#endif				/* #ifdef HAVE_ZLIB_H */
+
+}
+
+static libspectrum_error
+rzx_write_snapshot( libspectrum_buffer *buffer, libspectrum_buffer *block_data,
+                    libspectrum_snap *snap, libspectrum_id_t snap_format,
+                    libspectrum_creator *creator, int compress )
+{
+  libspectrum_error error = LIBSPECTRUM_ERROR_NONE;
   int flags, done;
   snapshot_string_t *type;
-
-  snap_length = 0;
+  libspectrum_buffer *snap_data = libspectrum_buffer_alloc();
+  size_t uncompressed_data_size;
 
   if( snap_format == LIBSPECTRUM_ID_UNKNOWN ) {
     /* If not given a snap format, try using .z80. If that would result
        in major information loss, use .szx instead */
     snap_format = LIBSPECTRUM_ID_SNAPSHOT_Z80;
-    error = libspectrum_snap_write( &snap_buffer, &snap_length, &flags, snap,
-				    snap_format, creator, 0 );
-    if( error ) return error;
+    error = libspectrum_snap_write_buffer( block_data, &flags, snap,
+                                           snap_format, creator, 0 );
+    if( error ) { goto cleanup; }
 
     if( flags & LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS ) {
-      libspectrum_free( snap_buffer ); snap_length = 0;
+      libspectrum_buffer_clear( block_data );
       snap_format = LIBSPECTRUM_ID_SNAPSHOT_SZX;
-      error = libspectrum_snap_write( &snap_buffer, &snap_length, &flags, snap,
-				      snap_format, creator, 0 );
-      if( error ) return error;
+      error = libspectrum_snap_write_buffer( block_data, &flags, snap,
+                                             snap_format, creator, 0 );
+      if( error ) { goto cleanup; }
     }
-
   } else {
-    error = libspectrum_snap_write( &snap_buffer, &snap_length, &flags, snap,
-				    snap_format, creator, 0 );
-    if( error ) return error;
+    error = libspectrum_snap_write_buffer( block_data, &flags, snap,
+                                           snap_format, creator, 0 );
+    if( error ) { goto cleanup; }
   }
 
   if( flags & LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS ) {
@@ -1402,185 +1446,95 @@ rzx_write_snapshot( libspectrum_byte **buffer, libspectrum_byte **ptr,
 			     __FILE__ );
   }
 
-  if( compress ) {
+  uncompressed_data_size = libspectrum_buffer_get_data_size( block_data );
+  rzx_compress( snap_data, block_data, &compress );
+  libspectrum_buffer_clear( block_data );
 
-#ifdef HAVE_ZLIB_H
-
-    error = libspectrum_zlib_compress( snap_buffer, snap_length,
-				       &gzsnap, &gzlength );
-    if( error != LIBSPECTRUM_ERROR_NONE ) {
-      libspectrum_free( snap_buffer );
-      return error;
-    }
-
-    libspectrum_make_room( buffer, 17 + gzlength, ptr, length );
-
-#else				/* #ifdef HAVE_ZLIB_H */
-
-    libspectrum_print_error( LIBSPECTRUM_ERROR_UNKNOWN,
-			     "rzx_write_snapshot: compression needs zlib" );
-    return LIBSPECTRUM_ERROR_UNKNOWN;
-
-#endif				/* #ifdef HAVE_ZLIB_H */
-
-  } else {
-    libspectrum_make_room( buffer, 17 + snap_length, ptr, length );
-  }
-
-  *(*ptr)++ = LIBSPECTRUM_RZX_SNAPSHOT_BLOCK;
-  if( compress ) {			/* Block length and flags */
-    libspectrum_write_dword( ptr, 17 + gzlength );
-    libspectrum_write_dword( ptr, 2 );
-  } else {
-    libspectrum_write_dword( ptr, 17 + snap_length );
-    libspectrum_write_dword( ptr, 0 );
-  }
+  libspectrum_buffer_write_dword( block_data, compress ? 2 : 0 );
 
   for( type = snapshot_strings, done = 0; type->format; type++ ) {
     if( type->format == snap_format ) {
-      memcpy( *ptr, type->string, 4 ); *ptr += 4;
+      libspectrum_buffer_write( block_data, type->string, 4 );
       done = 1;
       break;
     }
   }
 
   if( !done ) {
+    error =  LIBSPECTRUM_ERROR_UNKNOWN;
     libspectrum_print_error(
-      LIBSPECTRUM_ERROR_UNKNOWN,
+      error,
       "%s:rzx_write_snapshot: unexpected snap type %d", __FILE__, snap_format
     );
-    return LIBSPECTRUM_ERROR_UNKNOWN;
+    goto cleanup;
   }
 
-  libspectrum_write_dword( ptr, snap_length );	/* Snapshot length */
+  /* Uncompressed snapshot length */
+  libspectrum_buffer_write_dword( block_data, uncompressed_data_size );
 
-  if( compress ) {
-    memcpy( *ptr, gzsnap, gzlength ); (*ptr) += gzlength;
-    libspectrum_free( gzsnap );
-  } else {
-    memcpy( *ptr, snap_buffer, snap_length ); (*ptr) += snap_length;
-  }
+  libspectrum_buffer_write_buffer( block_data, snap_data );
 
-  libspectrum_free( snap_buffer );
+  rzx_write_block_header( buffer, block_data, LIBSPECTRUM_RZX_SNAPSHOT_BLOCK );
+
+cleanup:
+  libspectrum_buffer_free( snap_data );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
 
-static libspectrum_error
-rzx_write_input( input_block_t *block, libspectrum_byte **buffer,
-		 libspectrum_byte **ptr, size_t *length, int compress )
+static void
+rzx_write_input( input_block_t *block, libspectrum_buffer *buffer,
+                 libspectrum_buffer *block_data, int compress )
 {
-  size_t i, size;
-  size_t length_offset, data_offset, flags_offset;
-  libspectrum_byte *length_ptr; 
-
-  libspectrum_make_room( buffer, 18, ptr, length );
-
-  *(*ptr)++ = LIBSPECTRUM_RZX_INPUT_BLOCK;
-
-  /* The length bytes: for uncompressed data, 18 for the block introduction
-     and 4 per frame; the number of bytes in every frame is added in below.
-     If compression is requested (and makes the data shorter), this will be
-     overwritten with the compressed length */
-  size = 18 + 4 * block->count;
-
-  /* Store where the length will be written, and skip over those bytes */
-  length_offset = *ptr - *buffer; (*ptr) += 4;
-
-  /* How many frames? */
-  libspectrum_write_dword( ptr, block->count );
-
-  /* Each frame has an undefined length, so write a zero */
-  *(*ptr)++ = 0;
-
-  /* T-state counter */
-  libspectrum_write_dword( ptr, block->tstates );
-
-  /* Flags */
-  flags_offset = *ptr - *buffer;
-  libspectrum_write_dword( ptr, compress ? 0x02 : 0 );
+  size_t i;
+  libspectrum_buffer *frame_data = libspectrum_buffer_alloc();
 
   /* Write the frames */
-  data_offset = *ptr - *buffer;
   for( i = 0; i < block->count; i++ ) {
 
     libspectrum_rzx_frame_t *frame = &block->frames[i];
 
-    libspectrum_make_room( buffer, 4, ptr, length );
-    libspectrum_write_word( ptr, frame->instructions );
+    libspectrum_buffer_write_word( block_data, frame->instructions );
 
     if( frame->repeat_last ) {
-      libspectrum_write_word( ptr, libspectrum_rzx_repeat_frame );
+      libspectrum_buffer_write_word( block_data, libspectrum_rzx_repeat_frame );
     } else {
-
-      size += frame->count;			/* Keep track of the size */
-
-      libspectrum_write_word( ptr, frame->count );
-
-      libspectrum_make_room( buffer, frame->count, ptr, length );
-      memcpy( *ptr, frame->in_bytes, frame->count ); (*ptr) += frame->count;
+      libspectrum_buffer_write_word( block_data, frame->count );
+      libspectrum_buffer_write( block_data, frame->in_bytes, frame->count );
     }
-  }
-
-  /* Write the length in */
-  length_ptr = *buffer + length_offset;
-  libspectrum_write_dword( &length_ptr, size ); length_ptr -= 4;
-
-  if( compress ) {
-
-#ifdef HAVE_ZLIB_H
-
-    /* Compress the data the simple way. Really, we should stream the data */
-    libspectrum_byte *gzsnap = NULL; size_t gzlength;
-    libspectrum_byte *data_ptr = *buffer + data_offset;
-    libspectrum_error error;
-
-    error = libspectrum_zlib_compress( data_ptr, *ptr - data_ptr,
-				       &gzsnap, &gzlength );
-    if( error != LIBSPECTRUM_ERROR_NONE ) return error;
-
-    if( (ptrdiff_t)gzlength >= *ptr - data_ptr ) { /* Compression made it
-						      bigger :-( */
-      *(*buffer + flags_offset) &= ~0x02; /* Clear `compressed' bit */
-    } else {
-      /* Write the compressed data in */
-      memcpy( data_ptr, gzsnap, gzlength );
-
-      /* Correct the length word and the buffer length */
-      libspectrum_write_dword( &length_ptr, 18 + gzlength );
-      *ptr = *buffer + data_offset + gzlength;
-    }
-
-    libspectrum_free( gzsnap );
-
-#else				/* #ifdef HAVE_ZLIB_H */
-
-    libspectrum_print_error( LIBSPECTRUM_ERROR_UNKNOWN,
-			     "rzx_write_input: compression needs zlib" );
-    return LIBSPECTRUM_ERROR_UNKNOWN;
-
-#endif				/* #ifdef HAVE_ZLIB_H */
 
   }
 
-  return LIBSPECTRUM_ERROR_NONE;
+  rzx_compress( frame_data, block_data, &compress );
+  libspectrum_buffer_clear( block_data );
+
+  /* How many frames? */
+  libspectrum_buffer_write_dword( block_data, block->count );
+
+  /* Each frame has an undefined length, so write a zero */
+  libspectrum_buffer_write_byte( block_data, 0 );
+
+  /* T-state counter */
+  libspectrum_buffer_write_dword( block_data, block->tstates );
+
+  /* Flags */
+  libspectrum_buffer_write_dword( block_data, compress ? 0x02 : 0 );
+
+  libspectrum_buffer_write_buffer( block_data, frame_data );
+
+  rzx_write_block_header( buffer, block_data, LIBSPECTRUM_RZX_INPUT_BLOCK );
+
+  libspectrum_buffer_clear( block_data );
+  libspectrum_buffer_free( frame_data );
 }
 
 static libspectrum_error
-rzx_write_signed_start( libspectrum_byte **buffer, libspectrum_byte **ptr,
-			size_t *length, libspectrum_rzx_dsa_key *key,
+rzx_write_signed_start( libspectrum_buffer *buffer,
+                        libspectrum_buffer *block_data,
+                        libspectrum_rzx_dsa_key *key,
 			libspectrum_creator *creator )
 {
 #ifdef HAVE_GCRYPT_H
-  
-  libspectrum_make_room( buffer, 13, ptr, length );
-
-  /* Block ID */
-  *(*ptr)++ = LIBSPECTRUM_RZX_SIGN_START_BLOCK;
-
-  /* Block length */
-  libspectrum_write_dword( ptr, 13 );
-
   /* Key ID */
   if( !key || !key->y || strlen( key->y ) < 8 ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_INVALID,
@@ -1588,27 +1542,29 @@ rzx_write_signed_start( libspectrum_byte **buffer, libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_INVALID;
   }
 
-  libspectrum_write_dword(
-    ptr, strtoul( &key->y[ strlen( key->y ) - 8 ], NULL, 16 )
+  libspectrum_buffer_write_dword(
+    block_data, strtoul( &key->y[ strlen( key->y ) - 8 ], NULL, 16 )
   );
 
   /* Week code */
   if( creator ) {
-    libspectrum_write_dword( ptr,
+    libspectrum_buffer_write_dword( block_data,
 			     libspectrum_creator_competition_code( creator ) );
   } else {
-    libspectrum_write_dword( ptr, 0 );
+    libspectrum_buffer_write_dword( block_data, 0 );
   }
 
+  rzx_write_block_header( buffer, block_data,
+                          LIBSPECTRUM_RZX_SIGN_START_BLOCK );
+
 #endif				/* #ifdef HAVE_GCRYPT_H */
 
   return LIBSPECTRUM_ERROR_NONE;
 }
 
 static libspectrum_error
-rzx_write_signed_end( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		      size_t *length, ptrdiff_t sign_offset,
-		      libspectrum_rzx_dsa_key *key )
+rzx_write_signed_end( libspectrum_buffer *buffer, libspectrum_buffer *block_data,
+                      libspectrum_rzx_dsa_key *key )
 {
 #ifdef HAVE_GCRYPT_H
   libspectrum_error error;
@@ -1616,23 +1572,19 @@ rzx_write_signed_end( libspectrum_byte **buffer, libspectrum_byte **ptr,
 
   /* Get the actual signature */
   error = libspectrum_sign_data( &signature, &sig_length,
-				 ( *buffer + sign_offset ),
-				 (*ptr) - ( *buffer + sign_offset ), key );
+                                 libspectrum_buffer_get_data( buffer ),
+                                 libspectrum_buffer_get_data_size( buffer ),
+                                 key );
   if( error ) return error;
 
-  libspectrum_make_room( buffer, sig_length + 5, ptr, length );
-
-  /* Block ID */
-  *(*ptr)++ = LIBSPECTRUM_RZX_SIGN_END_BLOCK;
-
-  /* Block length */
-  libspectrum_write_dword( ptr, sig_length + 5 );
-
   /* Write the signature */
-  memcpy( *ptr, signature, sig_length ); (*ptr) += sig_length;
+  libspectrum_buffer_write( block_data, signature, sig_length );
 
   libspectrum_free( signature );
 
+  rzx_write_block_header( buffer, block_data,
+                          LIBSPECTRUM_RZX_SIGN_END_BLOCK );
+
 #endif				/* #ifdef HAVE_GCRYPT_H */
 
   return LIBSPECTRUM_ERROR_NONE;
