@@ -31,8 +31,6 @@
 #define LIBSPECTRUM_SNA_HEADER_LENGTH 27
 #define LIBSPECTRUM_SNA_128_HEADER_LENGTH 4
 
-#define SNA_OFFSET_SP 23
-
 static int identify_machine( size_t buffer_length, libspectrum_snap *snap );
 static int libspectrum_sna_read_header( const libspectrum_byte *buffer,
 					size_t buffer_length,
@@ -48,16 +46,15 @@ static int libspectrum_sna_read_128_data( const libspectrum_byte *buffer,
 					  libspectrum_snap *snap );
 
 static void
-write_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
-	      size_t *length, libspectrum_snap *snap );
+write_header( libspectrum_buffer *buffer, libspectrum_snap *snap,
+              libspectrum_word sp );
 static libspectrum_error
-write_48k_sna( libspectrum_byte **buffer, libspectrum_byte **ptr,
-	       size_t *length, libspectrum_snap *snap );
-static libspectrum_error
-write_128k_sna( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		size_t *length, libspectrum_snap *snap );
-static libspectrum_error
-write_page( libspectrum_byte *buffer, libspectrum_snap *snap, int page );
+write_48k_sna( libspectrum_buffer *buffer, libspectrum_snap *snap,
+               libspectrum_word *new_sp );
+static void
+write_128k_sna( libspectrum_buffer *buffer, libspectrum_snap *snap );
+static void
+write_page( libspectrum_buffer *buffer, libspectrum_snap *snap, int page );
 
 libspectrum_error
 internal_sna_read( libspectrum_snap *snap,
@@ -276,12 +273,12 @@ libspectrum_sna_read_128_data( const libspectrum_byte *buffer,
 }
 
 libspectrum_error
-libspectrum_sna_write( libspectrum_byte **buffer, size_t *length,
-		       int *out_flags, libspectrum_snap *snap,
-		       int in_flags GCC_UNUSED )
+libspectrum_sna_write( libspectrum_buffer *buffer, int *out_flags,
+                       libspectrum_snap *snap, int in_flags GCC_UNUSED )
 {
+  libspectrum_word snap_sp;
+  libspectrum_buffer *buffer_mem;
   libspectrum_error error = LIBSPECTRUM_ERROR_NONE;
-  libspectrum_byte *ptr;
 
   /* Minor info loss already due to things like tstate count, halted state,
      etc which are not stored in .sna format */
@@ -346,9 +343,15 @@ libspectrum_sna_write( libspectrum_byte **buffer, size_t *length,
   if( libspectrum_snap_didaktik80_active( snap ) )
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
 
-  ptr = *buffer;
+  /* We don't save the Covox state at all */
+  if( libspectrum_snap_covox_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
 
-  write_header( buffer, &ptr, length, snap );
+  /* We don't save the Multiface state at all */
+  if( libspectrum_snap_multiface_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+
+  buffer_mem = libspectrum_buffer_alloc();
 
   switch( libspectrum_snap_machine( snap ) ) {
 
@@ -360,7 +363,7 @@ libspectrum_sna_write( libspectrum_byte **buffer, size_t *length,
     /* Fall through */
   case LIBSPECTRUM_MACHINE_16:
   case LIBSPECTRUM_MACHINE_48:
-    error = write_48k_sna( buffer, &ptr, length, snap );
+    error = write_48k_sna( buffer_mem, snap, &snap_sp );
     break;
     
   case LIBSPECTRUM_MACHINE_128:
@@ -376,59 +379,64 @@ libspectrum_sna_write( libspectrum_byte **buffer, size_t *length,
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
     /* Fall through */
   case LIBSPECTRUM_MACHINE_PENT:
-    error = write_128k_sna( buffer, &ptr, length, snap );
+    snap_sp = libspectrum_snap_sp ( snap );
+    write_128k_sna( buffer_mem, snap );
     break;
 
-  case LIBSPECTRUM_MACHINE_UNKNOWN:
+  default:
     libspectrum_print_error( LIBSPECTRUM_ERROR_LOGIC,
 			     "Emulated machine type is set to 'unknown'!" );
+    libspectrum_buffer_free( buffer_mem );
     return LIBSPECTRUM_ERROR_LOGIC;
   }
 
-  if( error ) return error;
+  if( error ) {
+    libspectrum_buffer_free( buffer_mem );
+    return error;
+  }
 
-  /* Set length to be actual length, not allocated length */
-  *length = ptr - *buffer;
+  write_header( buffer, snap, snap_sp );
+  libspectrum_buffer_write_buffer( buffer, buffer_mem );
+  libspectrum_buffer_free( buffer_mem );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
 
 static void
-write_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
-	      size_t *length, libspectrum_snap *snap )
+write_header( libspectrum_buffer *buffer, libspectrum_snap *snap,
+              libspectrum_word sp )
 {
-  libspectrum_make_room( buffer, LIBSPECTRUM_SNA_HEADER_LENGTH, ptr, length );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_i  ( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_hl_( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_de_( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_bc_( snap ) );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_f_ ( snap ) );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_a_ ( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_hl ( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_de ( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_bc ( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_iy ( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_ix ( snap ) );
 
-  *(*ptr)++ = libspectrum_snap_i ( snap );
-  libspectrum_write_word( ptr, libspectrum_snap_hl_( snap ) );
-  libspectrum_write_word( ptr, libspectrum_snap_de_( snap ) );
-  libspectrum_write_word( ptr, libspectrum_snap_bc_( snap ) );
-  *(*ptr)++ = libspectrum_snap_f_( snap );
-  *(*ptr)++ = libspectrum_snap_a_( snap );
-  libspectrum_write_word( ptr, libspectrum_snap_hl ( snap ) );
-  libspectrum_write_word( ptr, libspectrum_snap_de ( snap ) );
-  libspectrum_write_word( ptr, libspectrum_snap_bc ( snap ) );
-  libspectrum_write_word( ptr, libspectrum_snap_iy ( snap ) );
-  libspectrum_write_word( ptr, libspectrum_snap_ix ( snap ) );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_iff2( snap ) ? 0x04 : 0x00 );
 
-  *(*ptr)++ = libspectrum_snap_iff2( snap ) ? 0x04 : 0x00;
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_r ( snap ) );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_f ( snap ) );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_a ( snap ) );
 
-  *(*ptr)++ = libspectrum_snap_r ( snap );
-  *(*ptr)++ = libspectrum_snap_f ( snap );
-  *(*ptr)++ = libspectrum_snap_a ( snap );
+  libspectrum_buffer_write_word( buffer, sp );
 
-  libspectrum_write_word( ptr, libspectrum_snap_sp ( snap ) );
-
-  *(*ptr)++ = libspectrum_snap_im( snap );
-  *(*ptr)++ = libspectrum_snap_out_ula( snap ) & 0x07;
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_im( snap ) );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_out_ula( snap ) & 0x07 );
 }
 
 static libspectrum_error
-write_48k_sna( libspectrum_byte **buffer, libspectrum_byte **ptr,
-	       size_t *length, libspectrum_snap *snap )
+write_48k_sna( libspectrum_buffer *buffer, libspectrum_snap *snap,
+               libspectrum_word *new_sp )
 {
-  libspectrum_error error;
-  libspectrum_byte *stack, *sp;
+  libspectrum_byte *stack, *memory_base;
+  size_t offset;
+  libspectrum_word new_stack_address;
 
   /* Must have somewhere in RAM to store PC */
   if( libspectrum_snap_sp( snap ) < 0x4002 ) {
@@ -438,75 +446,59 @@ write_48k_sna( libspectrum_byte **buffer, libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_INVALID;
   }
 
-  libspectrum_make_room( buffer, 0xc000, ptr, length );
+  offset = libspectrum_buffer_get_data_size( buffer );
 
-  error = write_page( &( (*ptr)[ 0x0000 ] ), snap, 5 );
-  if( error ) return error;
-  error = write_page( &( (*ptr)[ 0x4000 ] ), snap, 2 );
-  if( error ) return error;
-  error = write_page( &( (*ptr)[ 0x8000 ] ), snap, 0 );
-  if( error ) return error;
+  write_page( buffer, snap, 5 );
+  write_page( buffer, snap, 2 );
+  write_page( buffer, snap, 0 );
+
+  memory_base = libspectrum_buffer_get_data( buffer ) + offset;
 
   /* Place PC on the stack */
-  stack = &( (*ptr)[ libspectrum_snap_sp( snap ) - 0x4000 - 2 ] );
+  new_stack_address = libspectrum_snap_sp( snap ) - 2;
+  stack = &( memory_base[ new_stack_address - 0x4000 ] );
   libspectrum_write_word( &stack, libspectrum_snap_pc( snap ) );
 
-  *ptr += 0xc000;
-
   /* Store the new value of SP */
-  sp = *buffer + SNA_OFFSET_SP;
-  libspectrum_write_word( &sp, libspectrum_snap_sp( snap ) - 2 );
+  *new_sp = new_stack_address;
 
   return LIBSPECTRUM_ERROR_NONE;
 }
 
-static libspectrum_error
-write_128k_sna( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		size_t *length, libspectrum_snap *snap )
+static void
+write_128k_sna( libspectrum_buffer *buffer, libspectrum_snap *snap )
 {
-  size_t i, page; libspectrum_error error;
+  size_t i, page;
   
   page = libspectrum_snap_out_128_memoryport( snap ) & 0x07;
 
-  libspectrum_make_room( buffer, 0xc000 + LIBSPECTRUM_SNA_128_HEADER_LENGTH,
-			 ptr, length );
+  write_page( buffer, snap, 5 );
+  write_page( buffer, snap, 2 );
+  write_page( buffer, snap, page );
 
-  error = write_page( *ptr, snap, 5 ); (*ptr) += 0x4000;
-  if( error ) return error;
-  error = write_page( *ptr, snap, 2 ); (*ptr) += 0x4000;
-  if( error ) return error;
-  error = write_page( *ptr, snap, page ); (*ptr) += 0x4000;
-  if( error ) return error;
-
-  libspectrum_write_word( ptr, libspectrum_snap_pc( snap ) );
-  *(*ptr)++ = libspectrum_snap_out_128_memoryport( snap );
-  *(*ptr)++ = '\0';
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_pc( snap ) );
+  libspectrum_buffer_write_byte( buffer,
+                                 libspectrum_snap_out_128_memoryport( snap ) );
+  libspectrum_buffer_write_byte( buffer, '\0' );
 
   for( i = 0; i < 8; i++ ) {
 
     /* Already written pages 5, 2 and whatever's paged in */
     if( i == 5 || i == 2 || i == page ) continue;
 
-    libspectrum_make_room( buffer, 0x4000, ptr, length );
-
-    error = write_page( *ptr, snap, i ); (*ptr) += 0x4000;
-    if( error ) return error;
+    write_page( buffer, snap, i );
   }
-
-  return LIBSPECTRUM_ERROR_NONE;
 }
 
-static libspectrum_error
-write_page( libspectrum_byte *buffer, libspectrum_snap *snap, int page )
+static void
+write_page( libspectrum_buffer *buffer, libspectrum_snap *snap, int page )
 {
   libspectrum_byte *ram;
 
   ram = libspectrum_snap_pages( snap, page );
   if( ram ) {
-    memcpy( buffer, ram, 0x4000 );
+    libspectrum_buffer_write( buffer, ram, 0x4000 );
   } else {
-    memset( buffer, 0xff, 0x4000 );
+    libspectrum_buffer_set( buffer, 0xff, 0x4000 );
   }
-
-  return LIBSPECTRUM_ERROR_NONE;
 }
