@@ -158,28 +158,24 @@ read_v2_block( const libspectrum_byte *buffer, libspectrum_byte **block,
 	       const libspectrum_byte *end );
 
 static libspectrum_error
-write_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
-	      size_t *length, int *flags, libspectrum_snap *snap );
+write_header( libspectrum_buffer *buffer, int *flags, libspectrum_snap *snap );
 static void
-write_base_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		   size_t *length, int *flags, libspectrum_snap *snap );
+write_base_header( libspectrum_buffer *buffer, int *flags,
+                   libspectrum_snap *snap );
 static libspectrum_error
-write_extended_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		       size_t *length, int *flags, libspectrum_snap *snap );
+write_extended_header( libspectrum_buffer *buffer, int *flags,
+                       libspectrum_snap *snap );
 static void
-write_pages( libspectrum_byte **buffer, libspectrum_byte **ptr,
-	     size_t *length, libspectrum_snap *snap, int compress );
+write_pages( libspectrum_buffer *buffer, libspectrum_snap *snap,
+             int compress );
 static void
-write_page( libspectrum_byte **buffer, libspectrum_byte **ptr,
-	    size_t *length, int page_num, libspectrum_byte *page,
+write_page( libspectrum_buffer *buffer, int page_num, libspectrum_byte *page,
 	    int compress );
 static void
-write_slt( libspectrum_byte **buffer, libspectrum_byte **ptr,
-	   size_t *length, libspectrum_snap *snap );
+write_slt( libspectrum_buffer *buffer, libspectrum_snap *snap );
 static void
-write_slt_entry( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		 size_t *length, libspectrum_word type, libspectrum_word id,
-		 libspectrum_dword slt_length );
+write_slt_entry( libspectrum_buffer *buffer, libspectrum_word type,
+                 libspectrum_word id, libspectrum_dword slt_length );
 
 static void
 compress_block( libspectrum_byte **dest, size_t *dest_length,
@@ -287,7 +283,7 @@ read_header( const libspectrum_byte *buffer, libspectrum_snap *snap,
     extra_header = buffer + LIBSPECTRUM_Z80_HEADER_LENGTH + 2;
 
     error = get_joystick_type( snap,
-                               extra_header + 32,
+                               extra_header + 31,
                                ( header[29] & 0xc0 ) >> 6,
                                *version
                              );
@@ -1115,11 +1111,10 @@ read_v2_block( const libspectrum_byte *buffer, libspectrum_byte **block,
 }
 
 libspectrum_error
-libspectrum_z80_write2( libspectrum_byte **buffer, size_t *length,
-			int *out_flags, libspectrum_snap *snap, int in_flags )
+libspectrum_z80_write2( libspectrum_buffer *buffer, int *out_flags,
+                        libspectrum_snap *snap, int in_flags )
 {
   libspectrum_error error;
-  libspectrum_byte *ptr = *buffer;
 
   *out_flags = 0;
 
@@ -1193,67 +1188,73 @@ libspectrum_z80_write2( libspectrum_byte **buffer, size_t *length,
   if( libspectrum_snap_didaktik80_active( snap ) )
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
 
-  error = write_header( buffer, &ptr, length, out_flags, snap );
+  /* .z80 format doesn't save the Covox state at all */
+  if( libspectrum_snap_covox_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+
+  /* .z80 format doesn't save the Multiface state well */
+  if( libspectrum_snap_multiface_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+
+  error = write_header( buffer, out_flags, snap );
   if( error != LIBSPECTRUM_ERROR_NONE ) return error;
 
   write_pages(
-    buffer, &ptr, length, snap,
+    buffer, snap,
     !( in_flags & LIBSPECTRUM_FLAG_SNAPSHOT_NO_COMPRESSION) +
     ( in_flags & LIBSPECTRUM_FLAG_SNAPSHOT_ALWAYS_COMPRESS )
   );
 
-  (*length) = ptr - *buffer;
-
   return LIBSPECTRUM_ERROR_NONE;
 }
 
 static libspectrum_error
-write_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
-	      size_t *length, int *flags, libspectrum_snap *snap )
+write_header( libspectrum_buffer *buffer, int *flags, libspectrum_snap *snap )
 {
   libspectrum_error error;
 
-  write_base_header( buffer, ptr, length, flags, snap );
+  write_base_header( buffer, flags, snap );
 
-  error = write_extended_header( buffer, ptr, length, flags, snap );
+  error = write_extended_header( buffer, flags, snap );
   if( error != LIBSPECTRUM_ERROR_NONE ) return error;
 
   return LIBSPECTRUM_ERROR_NONE;
 }
 
 static void
-write_base_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		   size_t *length, int *flags, libspectrum_snap *snap )
+write_base_header( libspectrum_buffer *buffer, int *flags,
+                   libspectrum_snap *snap )
 {
   libspectrum_byte joystick_flags = 0;
 
-  libspectrum_make_room( buffer, LIBSPECTRUM_Z80_HEADER_LENGTH, ptr, length );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_a( snap ) ); 
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_f( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_bc( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_hl( snap ) );
 
-  *(*ptr)++ = libspectrum_snap_a( snap ); 
-  *(*ptr)++ = libspectrum_snap_f( snap );
-  libspectrum_write_word( ptr, libspectrum_snap_bc( snap ) );
-  libspectrum_write_word( ptr, libspectrum_snap_hl( snap ) );
+  libspectrum_buffer_write_word( buffer, 0 ); /* PC; zero denotes >= v2 */
 
-  libspectrum_write_word( ptr, 0 ); /* PC; zero denotes >= v2 */
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_sp( snap ) );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_i( snap ) );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_r( snap ) & 0x7f );
 
-  libspectrum_write_word( ptr, libspectrum_snap_sp( snap ) );
-  *(*ptr)++ = libspectrum_snap_i( snap );
-  *(*ptr)++ = ( libspectrum_snap_r( snap ) & 0x7f );
+  libspectrum_buffer_write_byte( buffer,
+              ( libspectrum_snap_r( snap ) >> 7 ) +
+              ( ( libspectrum_snap_out_ula( snap ) & 0x07 ) << 1 ) );
 
-  *(*ptr)++ = ( libspectrum_snap_r( snap ) >> 7 ) +
-              ( ( libspectrum_snap_out_ula( snap ) & 0x07 ) << 1 );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_de( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_bc_( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_de_( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_hl_( snap ) );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_a_( snap ) );
+  libspectrum_buffer_write_byte( buffer, libspectrum_snap_f_( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_iy( snap ) );
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_ix( snap ) );
 
-  libspectrum_write_word( ptr, libspectrum_snap_de( snap ) );
-  libspectrum_write_word( ptr, libspectrum_snap_bc_( snap ) );
-  libspectrum_write_word( ptr, libspectrum_snap_de_( snap ) );
-  libspectrum_write_word( ptr, libspectrum_snap_hl_( snap ) );
-  *(*ptr)++ = libspectrum_snap_a_( snap );
-  *(*ptr)++ = libspectrum_snap_f_( snap );
-  libspectrum_write_word( ptr, libspectrum_snap_iy( snap ) );
-  libspectrum_write_word( ptr, libspectrum_snap_ix( snap ) );
-
-  *(*ptr)++ = libspectrum_snap_iff1( snap ) ? 0xff : 0x00;
-  *(*ptr)++ = libspectrum_snap_iff2( snap ) ? 0xff : 0x00;
+  libspectrum_buffer_write_byte( buffer,
+                                 libspectrum_snap_iff1( snap ) ? 0xff : 0x00 );
+  libspectrum_buffer_write_byte( buffer,
+                                 libspectrum_snap_iff2( snap ) ? 0xff : 0x00 );
 
   /* Exactly one active joystick can be represented in a .z80 file */
   if( libspectrum_snap_joystick_active_count( snap ) != 1 )
@@ -1284,16 +1285,17 @@ write_base_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
   }
   joystick_flags <<= 6;
 
-  *(*ptr)++ = ( libspectrum_snap_im( snap ) & 0x03 ) +
+  libspectrum_buffer_write_byte( buffer,
+              ( libspectrum_snap_im( snap ) & 0x03 ) +
               ( libspectrum_snap_issue2( snap ) ? 0x04 : 0x00 ) +
-              joystick_flags;
+              joystick_flags );
 }
 
 static libspectrum_error
-write_extended_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		       size_t *length, int *flags, libspectrum_snap *snap )
+write_extended_header( libspectrum_buffer *buffer, int *flags,
+                       libspectrum_snap *snap )
 {
-  int i, bottom_16kb_ram;
+  int i, first_8kb_ram, second_8kb_ram;
   libspectrum_byte hardware_flag = 0;      /* No special emulation features */
   int second_memory_port = 0; /* By default, don't write the extra bytes */
   libspectrum_byte machine_byte = Z80_MACHINE_48;
@@ -1377,32 +1379,32 @@ write_extended_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
   header_length = second_memory_port ? LIBSPECTRUM_Z80_V3X_LENGTH
                                      : LIBSPECTRUM_Z80_V3_LENGTH;
 
-  /* +2 here to deal with the two length bytes */
-  libspectrum_make_room( buffer, header_length + 2, ptr, length );
+  libspectrum_buffer_write_word( buffer, header_length );
 
-  libspectrum_write_word( ptr, header_length );
-
-  libspectrum_write_word( ptr, libspectrum_snap_pc( snap ) );
-  *(*ptr)++ = machine_byte;
+  libspectrum_buffer_write_word( buffer, libspectrum_snap_pc( snap ) );
+  libspectrum_buffer_write_byte( buffer, machine_byte );
 
   if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_128_MEMORY ) {
-    *(*ptr)++ = libspectrum_snap_out_128_memoryport( snap );
+    libspectrum_buffer_write_byte( buffer,
+                                 libspectrum_snap_out_128_memoryport( snap ) );
   } else if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_MEMORY ) {
-    *(*ptr)++ = libspectrum_snap_out_scld_hsr( snap );
+    libspectrum_buffer_write_byte( buffer,
+                                   libspectrum_snap_out_scld_hsr( snap ) );
   } else {
-    *(*ptr)++ = '\0';
+    libspectrum_buffer_write_byte( buffer, '\0' );
   }
 
   if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_VIDEO ) {
-    *(*ptr)++ = libspectrum_snap_out_scld_dec( snap );
+    libspectrum_buffer_write_byte( buffer,
+                                   libspectrum_snap_out_scld_dec( snap ) );
     if( libspectrum_snap_interface1_active( snap ) &&
         libspectrum_snap_interface1_paged( snap ) )
       *flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
   } else if( libspectrum_snap_interface1_active( snap ) &&
              libspectrum_snap_interface1_paged( snap ) ) {
-    *(*ptr)++ = 0xff;
+    libspectrum_buffer_write_byte( buffer, 0xff );
   } else {
-    *(*ptr)++ = '\0';
+    libspectrum_buffer_write_byte( buffer, '\0' );
   }
 
   /* Support 16K snapshots via Spectaculator's extension; see the
@@ -1423,98 +1425,108 @@ write_extended_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
     hardware_flag |= melodik_flags;
   }
 
-  *(*ptr)++ = hardware_flag;
+  libspectrum_buffer_write_byte( buffer, hardware_flag );
 
-  *(*ptr)++ = libspectrum_snap_out_ay_registerport( snap );
+  libspectrum_buffer_write_byte( buffer,
+                               libspectrum_snap_out_ay_registerport( snap ) );
   for( i = 0; i < 16; i++ )
-    *(*ptr)++ = libspectrum_snap_ay_registers( snap, i );
+    libspectrum_buffer_write_byte( buffer,
+                                   libspectrum_snap_ay_registers( snap, i ) );
 
   quarter_states =
     libspectrum_timings_tstates_per_frame(
       libspectrum_snap_machine( snap ) 
     ) / 4;
 
-  libspectrum_write_word(
-    ptr,
+  libspectrum_buffer_write_word(
+    buffer,
     quarter_states - ( libspectrum_snap_tstates( snap ) % quarter_states ) - 1
   );
 
-  *(*ptr)++ =
-    ( ( libspectrum_snap_tstates( snap ) / quarter_states ) + 3 ) % 4;
+  libspectrum_buffer_write_byte( buffer,
+    ( ( libspectrum_snap_tstates( snap ) / quarter_states ) + 3 ) % 4 );
 
   /* Spectator disabled */
-  *(*ptr)++ = '\0';
+  libspectrum_buffer_write_byte( buffer, '\0' );
 
   /* MGT only for +D for now */
-  if( libspectrum_snap_plusd_active( snap ) ) {
-    *(*ptr)++ = libspectrum_snap_plusd_paged( snap );
+  if( libspectrum_snap_plusd_active( snap ) &&
+      libspectrum_snap_plusd_paged( snap ) ) {
+    libspectrum_buffer_write_byte( buffer, 0xff );
   } else {
-    *(*ptr)++ = '\0';
+    libspectrum_buffer_write_byte( buffer, 0x00 );
   }
 
   /* Multiface disabled */
-  *(*ptr)++ = '\0';
+  libspectrum_buffer_write_byte( buffer, '\0' );
 
   /* Is 0x0000 to 0x3fff RAM? True if we're in a +3 64Kb RAM
-     configuration, or a Scorpion configuration with page 0 mapped in */
-  bottom_16kb_ram = 0;
+     configuration, or a Scorpion configuration with page 0 mapped in.
+     +D and DISCiPLE also map RAM pages */
+  first_8kb_ram = second_8kb_ram = 0;
 
   if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_PLUS3_MEMORY ) {
 
-    if( libspectrum_snap_out_plus3_memoryport( snap ) & 0x01 )
-      bottom_16kb_ram = 1;
+    if( libspectrum_snap_out_plus3_memoryport( snap ) & 0x01 ) {
+      first_8kb_ram = 1;
+      second_8kb_ram = 1;
+    }
 
   } else if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_SCORP_MEMORY ) {
 
-    if( libspectrum_snap_out_plus3_memoryport( snap ) & 0x01 )
-      bottom_16kb_ram = 1;
+    if( libspectrum_snap_out_plus3_memoryport( snap ) & 0x01 ) {
+      first_8kb_ram = 1;
+      second_8kb_ram = 1;
+    }
+  }
+  else if( libspectrum_snap_plusd_active( snap ) &&
+           libspectrum_snap_plusd_paged( snap ) ) {
+    first_8kb_ram = 0;
+    second_8kb_ram = 1;
   }
 
-  if( bottom_16kb_ram ) {
-    *(*ptr)++ = 0xff; *(*ptr)++ = 0xff;
-  } else {
-    *(*ptr)++ = 0x00; *(*ptr)++ = 0x00;
-  }
+  libspectrum_buffer_write_byte( buffer, first_8kb_ram ? 0x00 : 0xff );
+  libspectrum_buffer_write_byte( buffer, second_8kb_ram ? 0x00 : 0xff );
 
   /* Joystick settings */
   if( libspectrum_snap_joystick_list( snap, 1 ) ==
         LIBSPECTRUM_JOYSTICK_SINCLAIR_2 ) {
     /* First 10 bytes are spectrum keyboard mappings for user defined keys
        in this order: left, right, down, up, fire */
-    libspectrum_write_word( ptr, if2_left_map_l );
-    libspectrum_write_word( ptr, if2_left_map_r );
-    libspectrum_write_word( ptr, if2_left_map_d );
-    libspectrum_write_word( ptr, if2_left_map_u );
-    libspectrum_write_word( ptr, if2_left_map_f );
+    libspectrum_buffer_write_word( buffer, if2_left_map_l );
+    libspectrum_buffer_write_word( buffer, if2_left_map_r );
+    libspectrum_buffer_write_word( buffer, if2_left_map_d );
+    libspectrum_buffer_write_word( buffer, if2_left_map_u );
+    libspectrum_buffer_write_word( buffer, if2_left_map_f );
     /* Second 10 bytes are ascii words corresponding to keys */
-    libspectrum_write_word( ptr, if2_left_l );
-    libspectrum_write_word( ptr, if2_left_r );
-    libspectrum_write_word( ptr, if2_left_d );
-    libspectrum_write_word( ptr, if2_left_u );
-    libspectrum_write_word( ptr, if2_left_f );
+    libspectrum_buffer_write_word( buffer, if2_left_l );
+    libspectrum_buffer_write_word( buffer, if2_left_r );
+    libspectrum_buffer_write_word( buffer, if2_left_d );
+    libspectrum_buffer_write_word( buffer, if2_left_u );
+    libspectrum_buffer_write_word( buffer, if2_left_f );
   } else {
-    for( i=32; i<52; i++ ) *(*ptr)++ = '\0';
+    for( i=0; i<20; i++ ) libspectrum_buffer_write_byte( buffer, '\0' );
   }
 
   /* MGT type */
   if( libspectrum_snap_plusd_active( snap ) ) {
-    *(*ptr)++ = Z80_MGT_PLUSD;
-    *(*ptr)++ = 0xff; /*    0-8191 ROM */
-    *(*ptr)++ = 0x00; /* 8192-9383 RAM */
+    libspectrum_buffer_write_byte( buffer, Z80_MGT_PLUSD );
+    libspectrum_buffer_write_byte( buffer, 0x00 ); /* DISCiPLE inhibitor */
+    libspectrum_buffer_write_byte( buffer, 0x00 ); /* DISCiPLE pageable */
   } else {
-    for( i=52; i<55; i++ ) *(*ptr)++ = '\0';
+    for( i=52; i<55; i++ ) libspectrum_buffer_write_byte( buffer, '\0' );
   }
 
   if( second_memory_port ) {
-    *(*ptr)++ = libspectrum_snap_out_plus3_memoryport( snap );
+    libspectrum_buffer_write_byte( buffer,
+                               libspectrum_snap_out_plus3_memoryport( snap ) );
   }
 
   return LIBSPECTRUM_ERROR_NONE;
 }
 
 static void
-write_pages( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
-	     libspectrum_snap *snap, int compress )
+write_pages( libspectrum_buffer *buffer, libspectrum_snap *snap, int compress )
 {
   int i;
   int do_slt;
@@ -1530,7 +1542,7 @@ write_pages( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
     memcpy( uncompressed, libspectrum_snap_interface1_rom( snap, 0 ),
             libspectrum_snap_interface1_rom_length( snap, 0 ) );
 
-    write_page( buffer, ptr, length, 1, uncompressed, compress );
+    write_page( buffer, 1, uncompressed, compress );
 
     libspectrum_free( uncompressed );
   }
@@ -1546,32 +1558,32 @@ write_pages( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
     memcpy( uncompressed + 0x2000,
             libspectrum_snap_plusd_ram( snap, 0 ), 0x2000 );
 
-    write_page( buffer, ptr, length, 1, uncompressed, compress );
+    write_page( buffer, 1, uncompressed, compress );
 
     libspectrum_free( uncompressed );
   }
 
   if( !( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_128_MEMORY ) ) {
 
-    write_page( buffer, ptr, length, 4, libspectrum_snap_pages( snap, 2 ),
+    write_page( buffer, 4, libspectrum_snap_pages( snap, 2 ),
                 compress );
-    write_page( buffer, ptr, length, 5, libspectrum_snap_pages( snap, 0 ),
+    write_page( buffer, 5, libspectrum_snap_pages( snap, 0 ),
                 compress );
-    write_page( buffer, ptr, length, 8, libspectrum_snap_pages( snap, 5 ),
+    write_page( buffer, 8, libspectrum_snap_pages( snap, 5 ),
                 compress );
 
   } else {
 
     for( i=0; i<8; i++ )
       if( libspectrum_snap_pages( snap, i ) )
-	write_page( buffer, ptr, length, i+3, libspectrum_snap_pages( snap, i ),
+	write_page( buffer, i+3, libspectrum_snap_pages( snap, i ),
                     compress );
 
     if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_SCORP_MEMORY )
       for( i = 8; i < 16; i++ )
         if( libspectrum_snap_pages( snap, i ) )
-          write_page( buffer, ptr, length, i + 3,
-                      libspectrum_snap_pages( snap, i ), compress );
+          write_page( buffer, i + 3, libspectrum_snap_pages( snap, i ),
+                      compress );
   }
 
   /* Do we want to write .slt data? Definitely if we've got a loading
@@ -1583,12 +1595,12 @@ write_pages( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
     if( libspectrum_snap_slt_length( snap, i ) ) do_slt = 1;
 
   /* Write the data if we've got any */
-  if( do_slt ) write_slt( buffer, ptr, length, snap );
+  if( do_slt ) write_slt( buffer, snap );
 }
 
 static void
-write_page( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
-	    int page_num, libspectrum_byte *page, int compress )
+write_page( libspectrum_buffer *buffer, int page_num, libspectrum_byte *page,
+            int compress )
 {
   libspectrum_byte *compressed = NULL; size_t compressed_length;
 
@@ -1600,18 +1612,15 @@ write_page( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
   if( !compress || ( !(compress & LIBSPECTRUM_FLAG_SNAPSHOT_ALWAYS_COMPRESS) &&
                       compressed_length >= 0x4000 ) ) {
 
-    libspectrum_make_room( buffer, 3 + 0x4000, ptr, length );
-    libspectrum_write_word( ptr, 0xffff );
-    *(*ptr)++ = page_num;
-    memcpy( *ptr, page, 0x4000 ); *ptr += 0x4000;
+    libspectrum_buffer_write_word( buffer, 0xffff );
+    libspectrum_buffer_write_byte( buffer, page_num );
+    libspectrum_buffer_write( buffer, page, 0x4000 );
 
   } else {
 
-    libspectrum_make_room( buffer, 3 + compressed_length, ptr, length );
-
-    libspectrum_write_word( ptr, compressed_length );
-    *(*ptr)++ = page_num;
-    memcpy( *ptr, compressed, compressed_length ); *ptr += compressed_length;
+    libspectrum_buffer_write_word( buffer, compressed_length );
+    libspectrum_buffer_write_byte( buffer, page_num );
+    libspectrum_buffer_write( buffer, compressed, compressed_length );
 
   }
 
@@ -1619,8 +1628,7 @@ write_page( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
 }
 
 static void
-write_slt( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
-	   libspectrum_snap *snap )
+write_slt( libspectrum_buffer *buffer, libspectrum_snap *snap )
 {
   int i;
 
@@ -1629,11 +1637,8 @@ write_slt( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
 
   size_t compressed_screen_length; libspectrum_byte* compressed_screen;
 
-  /* Make room for the .slt signature */
-  libspectrum_make_room( buffer, slt_signature_length, ptr, length );
-
-  memcpy( *ptr, slt_signature, slt_signature_length );
-  *ptr += slt_signature_length;
+  libspectrum_buffer_write( buffer, slt_signature, slt_signature_length );
+  libspectrum_buffer_write_byte( buffer, slt_signature_length );
 
   /* Now write out the .slt directory, compressing the data along
      the way */
@@ -1647,7 +1652,7 @@ write_slt( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
             	      libspectrum_snap_slt( snap, i ),
 		      libspectrum_snap_slt_length( snap, i ) );
 
-      write_slt_entry( buffer, ptr, length, LIBSPECTRUM_SLT_TYPE_LEVEL, i,
+      write_slt_entry( buffer, LIBSPECTRUM_SLT_TYPE_LEVEL, i,
 		       compressed_length[i] );
     }
   }
@@ -1666,28 +1671,26 @@ write_slt( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
     }
 
     /* Write the directory entry */
-    write_slt_entry( buffer, ptr, length, LIBSPECTRUM_SLT_TYPE_SCREEN,
+    write_slt_entry( buffer, LIBSPECTRUM_SLT_TYPE_SCREEN,
 		     libspectrum_snap_slt_screen_level( snap ),
 		     compressed_screen_length );
   }
 
   /* and the directory end marker */
-  write_slt_entry( buffer, ptr, length, LIBSPECTRUM_SLT_TYPE_END, 0, 0 );
+  write_slt_entry( buffer, LIBSPECTRUM_SLT_TYPE_END, 0, 0 );
 
   /* Then write the actual level data */
   for( i=0; i<256; i++ ) {
     if( libspectrum_snap_slt_length( snap, i ) ) {
-      libspectrum_make_room( buffer, compressed_length[i], ptr, length );
-      memcpy( *ptr, compressed_data[i], compressed_length[i] );
-      *ptr += compressed_length[i];
+      libspectrum_buffer_write( buffer, compressed_data[i],
+                                compressed_length[i] );
     }
   }
 
   /* And the loading screen */
   if( libspectrum_snap_slt_screen( snap ) ) {
-    libspectrum_make_room( buffer, compressed_screen_length, ptr, length );
-    memcpy( *ptr, compressed_screen, compressed_screen_length );
-    *ptr += compressed_screen_length;
+    libspectrum_buffer_write( buffer, compressed_screen,
+                              compressed_screen_length );
   }
 
   /* Free up the compressed data */
@@ -1698,15 +1701,13 @@ write_slt( libspectrum_byte **buffer, libspectrum_byte **ptr, size_t *length,
 }
 
 static void
-write_slt_entry( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		 size_t *length, libspectrum_word type, libspectrum_word id,
-		 libspectrum_dword slt_length )
+write_slt_entry( libspectrum_buffer *buffer, libspectrum_word type,
+                 libspectrum_word id, libspectrum_dword slt_length )
 {
-  libspectrum_make_room( buffer, 8, ptr, length );
-  libspectrum_write_word( ptr, type );
-  libspectrum_write_word( ptr, id );
-  libspectrum_write_word( ptr, slt_length & 0xffff );
-  libspectrum_write_word( ptr, slt_length >> 16 );
+  libspectrum_buffer_write_word( buffer, type );
+  libspectrum_buffer_write_word( buffer, id );
+  libspectrum_buffer_write_word( buffer, slt_length & 0xffff );
+  libspectrum_buffer_write_word( buffer, slt_length >> 16 );
 }
 
 static void
